/* test driver for wrpQueue package */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "wrpqueue.h"

#define QUEUE_QUIT	0
#define QUEUE_ENQ		1
#define QUEUE_DEQ		2
#define QUEUE_PEEK	3
#define QUEUE_LIST	4

char *Desc[] = {
"Quit program",	/* queue_QUIT */
"Enqueue value on end of queue",    /* queue_PUSH */
"Dequeue value from front of queue",	/* queue_POP */
"Look at front of queue\n",			/* queue_PEEK */
"List contents of queue\n",		/* queue_LIST */
NULL};

struct cmd {
	char *str;	/* command string */
   int num;		/* command number */
};

static struct cmd Cmd[] = {
{ "enqueue", QUEUE_ENQ},
{ "dequeue", QUEUE_DEQ},
{ "peek", QUEUE_PEEK},
{ "list", QUEUE_LIST},
{ "enq", QUEUE_ENQ},
{ "deq", QUEUE_DEQ},
{ ">", QUEUE_ENQ},
{ "<", QUEUE_DEQ},
{ "^", QUEUE_PEEK},
{ "!", QUEUE_LIST},
{NULL, NULL}};

void do_help(void);
void list_ent(void *ent);

static int qcnt = 0;

int
main(int argc, char **argv)
{
	void * queue;
   char buf[BUFSIZ];
   int num, i;
   char *p, *str, *val;

   printf("Creating queue ...\n");
   queue = wrpQueue_create();

   /* main loop - get command and do it */
	while (1)
   {
   	printf("%d items on queue.\n", wrpQueue_count(queue));
		printf("Enter command, or ? for help: ");
      gets(buf);
      if (*buf == 'q') break;
      if (*buf == '?')
      {
      	do_help();
         continue;
      }

      /* a command and value should have been entered - parse string */
      for (p=buf; isspace(*p); p++);	/* find non-blank char */
      str = p;	/* this is the command string */
      while (*p != '\0' && !isspace(*p)) p++;	/* find end of string */
      if (*p != '\0') *p++ = '\0';	/* terminate command string */
      /* now skip past spaces to find start of value */
		while (*p != '\0' && isspace(*p)) p++;	/* find value */
      val = p;

      /* we now have a command string and a value - find match*/
		for (i=0; Cmd[i].str != NULL && strcmp(str, Cmd[i].str); i++);
      if (Cmd[i].str == NULL) printf("Command not found (%s)\n", str);
      else switch(Cmd[i].num)
      { /* process command */
      case QUEUE_ENQ:	/* push value */
      	/* we have to make a copy of what we push to the queue */
      	num = wrpQueue_enqueue(queue, strdup(val));	/* push value on queue */
         if (num != 0) printf("queue error (%d)\n", num);
         break;

      case QUEUE_DEQ:	/* pop value from queue */
      	str = wrpQueue_dequeue(queue);
         if (str == NULL) printf("queue error\n");
         else
         {
         	printf("queue returned >>%s<<\n", str);
            free(str);
         }
         break;

      case QUEUE_PEEK:	/* look at top of queue */
      	str = (char *)wrpQueue_peek(queue);
         if (str == NULL) printf("queue error\n");
         else printf("Top of queue contains >>%s<<\n", str);
         break;

      case QUEUE_LIST:	/* list entire queue */
      	qcnt = 0;
      	num = wrpQueue_list(queue, list_ent);
         if (num != 0) printf("queue error (%d)\n", num);
         break;

      default:
      	printf("Unknown command (%s)\n", str);
      }
   }

   printf("Freeing up queue ...\n");
   wrpQueue_free(queue);

   printf("Press ENTER to continue: ");
   gets(buf);
   return 0;
}

void do_help(void)
{ /* display help message */
	int i;

	puts("This program is a test client for the wrpQueue routines");
   puts("Enter a command and a value.  The value can be any string");
	puts("Valid commands are:");

   puts("Cmd\tDescription");
   for (i=0; Cmd[i].str != NULL; i++)
   { /* list commands */
   	printf("%s\t%s\n", Cmd[i].str, Desc[Cmd[i].num]);
   }
}

void list_ent(void *ent)
{
	char *str;

   if (ent == NULL) return;	/* bad arg */
   str = (char *)ent;
   printf("Entry %d >>%s<<\n", qcnt++, str);
}

