/*
 *				**** WRP Queue ****
 *
 * This file contains the implementation of a queue
 * using the LinkList package
 */

#include <stdio.h>
#include <stdlib.h>

#include "linklist.h"

static int debugsw = 0;

static void queue_list(int loc, void *ent);

void wrpQueue_debug(int val) { debugsw = val; }

void *wrpQueue_create(void)
{
	LinkList *list;

   list = create_link_list();

   return (void *)list;
}

int wrpQueue_free(void *queue)
{
	/*
    * there isn't a call to free a linked list
    * (there should be, but I guess I never wrote one)
    * so we will do the free ourselves
    */

   if (queue == NULL) return 1;	/* bad arg */
	free(queue);
   return 0;
}

int wrpQueue_enqueue(void *queue, void *ent)
{ /* add entry to end of queue */
	LinkList *list;
   Link *link;

   if (queue == NULL || ent == NULL) return 1; /* bad args */
   list = (LinkList *)queue;

   link = create_link_entry(ent);	/* create link for data */
   if (link == NULL) return 2; /* out of memory */
   add_link_to_tail(list, link);
   return 0;
}

void *wrpQueue_dequeue(void *queue)
{ /* remove entry from beg of queue */
	LinkList *list;
   Link *link;
   void *ent;

   if (queue == NULL) return NULL;	/* bad args */
   list = (LinkList *)queue;
   if (list_is_empty(list)) return NULL; /* nothing there */
   link = get_head_link(list);	/* get head link */
   ent = delete_link(list, link);	/* remove it & get data */

   return ent;
}

void *wrpQueue_peek(queue)
{
	LinkList *list;
   void *ent;

   if (queue == NULL) return NULL;
   list = (LinkList *)queue;
   ent = get_data_from_head(list);
   return ent;
}

int wrpQueue_count(void *queue)
{
	LinkList *list;
   int cnt = 0;

   if (queue == NULL) return -1;	/* bad arg */
	list = (LinkList *)queue;
   if (list_is_empty(list)) return 0;	/* nothing in queue */

   /* count entries in queue */
   set_head_current_link(list);	/* start at front of link */
   do
   {
   	cnt++;
      set_current_link_next(list);	/* move current to next entry */
   } while (! current_is_tail(list)); /* until end of list */

   return cnt;
}

int wrpQueue_list(void *queue, void (qlist)(void *ent))
{
	LinkList *list;
   void *ent;

   if (queue == NULL || qlist == NULL) return 1;	/* bad args */
   list = (LinkList *)queue;

   if (list_is_empty(list)) return 2;	/* nothing in queue */

   /* count entries in queue */
   set_head_current_link(list);	/* start at front of link */
   while (1)
   {
   	ent = get_data_from_current(list);
      qlist(ent);
      if (current_is_tail(list)) break;	/* all done */
      set_current_link_next(list);	/* move current to next entry */
   }

	return 0;
}

